import pygame
from math import floor
from random import choice, randrange

# Init
pygame.init()

# Util Functions
def draw_background():
    pygame.draw.rect(screen, GREEN, [0, 468, 1366, 300], 0)
    pygame.draw.rect(screen, LIGHT_BLUE, [0, 0, 1366, 468])
    screen.blit(sun, (1000, 100))
    

class Grack:
    def __init__(self, xpos = 0, ypos = 0):
        self.xpos = xpos
        self.ypos = ypos

        self.yvel = 0

        self.ground_pos = ypos
        self.grounded = True

        self.image = pygame.image.load("./img/grack.png").convert_alpha()

    def draw(self):
        screen.blit(self.image, (self.xpos, self.ypos))

    def tick_gravity(self):
        if not self.grounded:
            self.yvel += 1

    def move(self):
        if not self.grounded:
            self.ypos += self.yvel

            if self.ypos >= self.ground_pos:
                self.ypos = self.ground_pos
                self.grounded = True

    def jump(self):
        if self.grounded:
            self.yvel = -22
            self.grounded = False

class Cryptid:
    def __init__(self, image_name, xpos = 0, ypos = 0, xvel = 0):
        self.xpos = xpos
        self.ypos = ypos

        self.xvel = xvel

        self.image = pygame.image.load(f"./img/{image_name}.png").convert_alpha()

    def draw(self):
        screen.blit(self.image, (self.xpos, self.ypos))

    def move(self):
        self.xpos += self.xvel

class LoseText:
    def __init__(self, text, xpos = 0, ypos = 0, visible = False):
        self.visible = visible
        self.duration = 0

        self.font = pygame.font.SysFont("arial", 80)
        self.text = self.font.render(text, True, WHITE)
        self.text_rect = self.text.get_rect()   

        self.text_rect.center = (xpos, ypos)     

    def lose(self):
        self.duration = 120
        self.visible = True

    def tick(self):
        if self.duration != 0:
            self.duration -= 1
        else:
            self.visible = False

    def render(self):
        if self.visible:
            screen.blit(self.text, self.text_rect)

class TimeText:
    def __init__(self):
        self.time = 0

        self.font = pygame.font.SysFont("arial", 60)
        self.text = self.font.render(f"Time: {self.time}", True, WHITE)
        self.text_rect = self.text.get_rect()

        self.text_rect.topleft = (10, 10)

    def tick(self):
        self.time = floor(frame_counter / 60)

        self.text = self.font.render(f"Time: {self.time}", True, WHITE)
    
    def render(self):
        screen.blit(self.text, self.text_rect)
    

class CryptidGenerator():
    def __init__(self, start_speed = -8):
        self.delay = 150
        self.start_speed = start_speed

    def generate_cryptid(self, xvel):
        names = [
            "keeplant",
            "gumpingle",
            "stack",
            "harrhy",
            "mitch",
            "nit",
            "eth",
            "barmond",
            "pyrouli",
            "staceyrage",
            "pogdan",
            "guligrimace"
        ]

        name = choice(names)

        cryptid = Cryptid(name, 1366, 550, xvel)

        return cryptid

    def tick(self):
        if self.delay != 0:
            self.delay -= 1
        else:
            self.delay = randrange(60, 150)

            speed = -floor((frame_counter / 600)) - 8
            cryptids.append(generator.generate_cryptid(speed))

# Colors
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
GREEN = (0, 200, 0)
LIGHT_BLUE = (0, 150, 255)
YELLOW = (200, 200, 0)

# Open window
size = (1366, 768)
screen = pygame.display.set_mode(size)
pygame.display.set_caption("Grack Jump")

# Music
pygame.mixer.init()
pygame.mixer.music.load("./polka.mp3")
pygame.mixer.music.set_volume(0.25)
pygame.mixer.music.play(-1)

# Fail Sound
fail_sound = pygame.mixer.Sound("./lose.mp3")
fail_sound.set_volume(0.25)

# Game Icon
icon = pygame.image.load('./img/sun.png')
scaled_icon = pygame.transform.scale(icon, (32, 32))
pygame.display.set_icon(scaled_icon)

# Grack
grack = Grack(100, 400)

# Sun
sun = pygame.image.load("./img/sun.png").convert_alpha()

# Main Loop
carryOn = True

cryptids = []
lose_text = LoseText("You Lost!", 683, 100)
time_text = TimeText()
generator = CryptidGenerator()

clock = pygame.time.Clock()
frame_counter = 0

while carryOn:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            carryOn = False
        elif event.type == pygame.KEYDOWN:
            if event.key == pygame.K_SPACE:
                grack.jump()
            elif event.key == pygame.K_ESCAPE:
                carryOn = False
    
    # Background
    draw_background()

    # Grack
    grack.tick_gravity()
    grack.move()
    grack.draw()

    # Generate Cryptids
    generator.tick()

    # Cryptids
    for cryptid in cryptids:
        cryptid.move()
        cryptid.draw()

    # Trim old cryptids off screen
    cryptids = [c for c in cryptids if c.xpos + c.image.get_width() > 0]

    # Collision
    for cryptid in cryptids:
        if grack.xpos < cryptid.xpos + cryptid.image.get_width() and \
        grack.xpos + grack.image.get_width() > cryptid.xpos and \
        grack.ypos < cryptid.ypos + cryptid.image.get_height() and \
        grack.image.get_height() + grack.ypos > cryptid.ypos:
            cryptids = []
            frame_counter = -1
            lose_text.lose()
            fail_sound.play()

    # Lose Text
    lose_text.tick()
    lose_text.render()

    # Time Text
    time_text.tick()
    time_text.render()

    pygame.display.flip()

    clock.tick(60)
    frame_counter += 1

pygame.quit()
